%=========================================================================%
% Bernanke, Gertler and Gilchrist (1999)
% Ambrogio Cesa-Bianchi, August 2012
%
% RBC version of BGG with:
% - CES utility
% - Capital (no labor)
% - Time-varying S (~ variance of idiosyncratic shock)
% - Sticky prices (Rotemberg)
% - Quadratic adjustment cost on investment
% - Inverse elasticity of labor supply (nu) is calibrated to get N=.3 in 
%   steady state. To get the correct value of nu: run the SS file, 
%   compute the SS values for all variables and back out nu from (4)
% - Consumption Entrepreneurs
%=========================================================================%

% Set the type of preferences (1=GHH, 2=NonSep, 3=Sep)
@#define preference = 1
options_.dynatol.f = 1e-4;
%------------------ Variables --------------------------------------------%
var K N I C rn omb rk nw S z infl mc Ce Y A W efp q GDP dflt dstr B rp L yk RL Rn Rk Rr wage WW;
varexo eA eS eW;

%------------------ Parameters -------------------------------------------%
parameters  S_ss mu gamma alpha delta beta rhoS ra nu rhoA StdeA eps theta 
            infl_ss rn_ss StdeS rhoW StdeW rhor rhopi rhoy phik tau ;

    S_ss    = 0.225;
    mu      = 0.25;
    gamma   = 0.985;
    alpha   = 0.3;
    delta   = 0.025;
    beta    = 0.994;
    ra      = 2;
    nu      = 1; 
    eps     = 10;
    theta   = 110;
    infl_ss = 1.005;
    rn_ss   = infl_ss/beta-1;
    rhoA    = 0.98; 
    StdeA   = 0.007;  
    rhoS    = 0.86;
    StdeS   = 0.023; 
    rhoW    = 0.88;
    StdeW   = 0.14;    
    rhor     = 0.8;     
    rhoy     = 0.25;      
    rhopi    = 1.8;    
    phik     = 30;
    tau      = 2.497529437991429;

model;
%------------------ Auxiliary expressions --------------------------------%

  % a. Distributional form of idiosyncratic shock
    # F       = normcdf((log(omb) + S(-1)^2 / 2) / S(-1));
    # G       = normcdf((log(omb) + S(-1)^2 / 2) / S(-1) - S(-1));
    # d       = mu * G  * yk * K(-1);
    # Fp1     = normcdf((log(omb(+1)) + S^2 / 2) / S);
    # Gp1     = normcdf((log(omb(+1)) + S^2 / 2) / S - S);
    # GAMMA   = omb*(1-F)+G;
    # GAMMAp1 = omb(+1)*(1-Fp1)+Gp1;
    # dFp1    = normpdf((log(omb(+1)) + S^2 / 2) / S) / omb(+1) / S;

  % b. Marginal utility of consumption and labor

    @#if preference == 1
      % GHH
        # Uc   = (C - tau*N^(1+nu))^(-ra);
        # Ucp1 = (C(+1) - tau*N(+1)^(1+nu))^(-ra);
        # Un   = -tau*(1+nu)*(N^nu)*(C - tau*N^(1+nu))^(-ra);
    @#else
        @#if preference == 2
          % Non Separable
            # Uc   = C^(-ra)*(1-N)^(tau*(1-ra));
            # Ucp1 = C(+1)^(-ra)*(1-N(+1))^(tau*(1-ra));
            # Un   = -tau*C^(1-ra)*(1-N)^(tau*(1-ra)-1);
        @#else
          % Separable
            # Uc   = C^(-ra);
            # Ucp1 = C(+1)^(-ra);
            # Un   = -tau*N^(nu);
        @#endif 
    @#endif 


  % C. Production function and marginal product of factors
    # Yk  = exp(A)*alpha*(K(-1)^(alpha-1))*(N^(1-alpha));
    # Yn  = exp(A)*(1-alpha)*(K(-1)^alpha)*(N^(-alpha));

%------------------ Equilibrium equations --------------------------------%

  % 1. resource constraint
    Y = C + Ce + I + d + theta/2*(infl/infl_ss -1)^2;
  
  % 2. Law of motion for capital
    K = (1 - delta) * K(-1) + I - (phik/2)*((I/K(-1)-delta)^2)*K(-1);
  
  % 3. Household FOC w.r.t risk-free bonds
    Uc = beta * (1+rn) * Ucp1 / infl(+1);

  % 4. Household FOC w.r.t hours
    mc*Yn = - Un / Uc;

  % 5. Household FOC w.r.t hours
    mc*Yk = z;

  % 6. Efficiency condition associated with the contract (rn, rk, omb)
    1 + rk(+1) = efp * (1+rn);
    efp = (1-Fp1) / ( (1-Fp1)*(GAMMAp1-mu*Gp1) + (1-GAMMAp1)*(1-Fp1-mu*omb*dFp1) );

  % 7. Definition of income of real entrepreneurs, q(1+rk)
    yk = z + q*((1 - delta) - (phik/2)*((I/K(-1)-delta)^2) + phik*(I/K(-1)-delta)*(I/K(-1))) ;
    yk = (q(-1)*(1 + rk) ) / infl;
    
  % 8. Zero profit condition
    nw(-1) = (1-((1+rk)/(1+rn(-1)))*(GAMMA-mu*G)) * q(-1) * K(-1);

  % 9. Law of motion of net worth
    nw = gamma*(1-GAMMA)*yk*K(-1);

  % 10. Phillips curve
    %(infl/infl_ss-1)*infl/infl_ss = beta*(Ucp1/Uc)*(infl(+1)/infl_ss-1)*infl(+1)/infl_ss + (Y*eps)/theta*(mc - (eps - 1)/eps);
    (infl/infl_ss-1)*infl/infl_ss = beta*(Ucp1/Uc)*(infl(+1)/infl_ss-1)*infl(+1)/infl_ss + (eps)/theta*(mc - (eps - 1)/eps) ;

  % 11. Monetary Policy rule
    (1+rn)/(1+rn_ss) = ((1+rn(-1))/(1+rn_ss))^rhor * ((infl/infl_ss)^rhopi * (Y/Y(-1))^rhoy)^(1-rhor);

  % 12. Price of capital
    q = ( 1-phik*(I/K(-1)-delta) )^(-1); 

  % 13. Consumption entrepreneurs
    Ce = (1-gamma)*(1-GAMMA)*yk*K(-1);

  % 14. Shocks 
    log(S/S_ss) = rhoS*log(S(-1)/S_ss) + StdeS*eS ;
    A = rhoA*A(-1) + exp(W)*StdeA*eA;
    W = rhoW*W(-1) + StdeW*eW;

  % d. Reporting variables 
    Y   = exp(A)*(K(-1)^alpha)*(N^(1-alpha));
    GDP = C + Ce + I;
    dflt = F;
    dstr = d;
    L = (q*K(-1))/nw;
    B = q*K(-1) - nw;
    Rn = 1 + rn;
            rp = (efp/omb)*(1-(nw/(q*K(-1))));
            RL = Rn+(rp-1);
    Rk = 1+rk;
    Rr = 1 + (Rn-infl);
    wage = mc*Yn;
    WW = exp(W);
end;


%------------------ Shocks -----------------------------------------------%
shocks;
    var eA = 1;
    var eS = 1; 
    var eW = 1; 
end;


%------------------ Steady State -----------------------------------------%
%steady;
%check;

%------------------ Solve & Clean ----------------------------------------%

stoch_simul(order=3, periods=0, nocorr, nomoments, nograph);
