clear all
load RESULTS

% Number of draws for 
ndraws = 500;
% Steps for simulation
steps = 500;
% Steps for IRFs
IRFsteps = 40;
% Number of end variables and number of shocks 
nendovar = size(M_.endo_names,1);
nexovar  = size(M_.exo_names,1);
% number of subplots
nplots = ceil(sqrt(nendovar));

%% Choose which type of IRF you want to plot
disp(' ')
disp('Choose the type of IRFs you want to compute:')
disp(' ')
disp('1 --> From ergodic mean (as in Fernandez-Villaverde et al 2011, AER)')
disp('2 --> From steady state (modified from Fernandez-Villaverde et al 2011, AER)')
disp('3 --> From a boom & bust')
irf_type = input('? ');


% Define which exogenous process to shock
disp('Which shock ?')
junk(1:nexovar,1)={': '};
disp(' ')
disp([num2str([1:nexovar]') char(junk) M_.exo_names])
whichShock = input('? ');
clear junk


switch irf_type

    
    
%% IRF a la Fernandez-Villaverde et al (2011, AER)
%==========================================================================
% Compute deterministic simulation from ergodic mean
%
% Higher order approximation makes the simulated paths of states and 
% controls in the model move away from their steady-state values. 
% Consequently, computing impulse responses as percentage deviations of 
% the model's steady state is not informative. To compute the impulse 
% responses reported in the paper, we proceed as follows:
% 
% 1. We simulate the model, starting from its steady state, for 2096 
% periods. We disregard the first 2000 periods as a burn-in
%
% 2. Based on the last 96 periods, we compute the mean of the ergodic 
% distribution for each variable in our model. Adding more periods has 
% essentially no impact on the mean 
%
% 3. Starting from the ergodic mean and in the absence of shocks, we hit 
% the model with a one-standard-deviation shock to volatility 
%
% 4. We report the resulting impulse responses as percentage deviations 
% from the variables' ergodic means.

    case 1

        % Draw random shocks and simulate the model
        exopath = randn(steps,nexovar);
        out_baseline = simult_(oo_.steady_state,oo_.dr,exopath,3);

        % Compute ergodic mean from the last 96 obs of the simulated model
%         ergo_mean = mean(out_baseline(:,end-95:end),2);
        ergo_mean = mean(out_baseline(:,ceil(steps/2):end),2); % use last half of the simulation

            % Compare ergo_mean and SS
            figure
            for ii=1:nendovar
                temp1(1:steps+1) = ergo_mean(ii);
                temp2(1:steps+1) = oo_.steady_state(ii);
                subplot(nplots,nplots,ii)
%                 plot(out_baseline(ii,ceil(steps/2):end),'-k','LineWidth',.1);
%                 hold on
                plot(temp2(ceil(steps/2):end),'--g','LineWidth',2);
                hold on
                plot(temp1(ceil(steps/2):end),':r','LineWidth',2);
%                 plot(zeros(IRFsteps),'-r','LineWidth',.5)
                title(M_.endo_names(ii,:));
                xlim( [0 ceil(steps/2)] )
            end
            LegendSubplot({'Steady state';'Ergodic mean'},.1)
            suptitle('Difference between Steady State and Ergodic Mean')
            set(gcf, 'Color', 'w');
            figure_name = 'SS_VS_ErgoMean';
            print(figure_name,'-dpdf'); print(figure_name,'-deps')
        
        
        % Initialize the path of the exogenous process 
        exopath_shock = zeros(IRFsteps,nexovar);

        % Set a 2 st. dev shock at period 1 for the chosen exogenous process
        exopath_shock(1,whichShock) = 2; % 2 st. dev. shock

        % Simulate the deterministic shock from the ergodic mean
        out = simult_(ergo_mean,oo_.dr,exopath_shock,3);

        % Compute percent deviations from the mean
        for ii=1:nendovar
%                 junk(ii,:,jj) = (out(ii,:) - ergo_mean(ii));
            IRF(ii,:) = (out(ii,:) - oo_.steady_state(ii));
        end

        figure
        nplots = ceil(sqrt(nendovar));
        for ii=1:nendovar
            subplot(nplots,nplots,ii)
            plot(IRF(ii,2:end),'-k','LineWidth',2);
            hold on
            plot(zeros(IRFsteps),'-r','LineWidth',.5)
            title(M_.endo_names(ii,:));
            xlim( [0 IRFsteps] )
        end
        suptitle('IRF Computed from Ergodic Mean')
        set(gcf, 'Color', 'w');
        figure_name = 'IRF_ErgoMean';
        print(figure_name,'-dpdf'); print(figure_name,'-deps')



%% 2. Alternative IRF: compute deterministic simulation from steady state
%==========================================================================
% As above but instead of using the ergodic mean, use the steady state 
% as an initial value. RESULTS ARE MUCH BETTER! 

    case 2

        % Initialize the path of shocks 
        exopath_shock = zeros(IRFsteps,nexovar);

        % Set a 2 st. dev shock at period 1 for the chosen exogenous process
        exopath_shock(1,whichShock) = 2; % 2 st. dev. shock

        % Simulate the deterministic shock from the steady state
        out = simult_(oo_.steady_state,oo_.dr,exopath_shock,3);

        % Compute deviations from the steady state
        for ii=1:nendovar
            IRF(ii,:) = (out(ii,:) - oo_.steady_state(ii));
        end

        figure
        nplots = ceil(sqrt(nendovar));
        for ii=1:nendovar
            subplot(nplots,nplots,ii)
            plot(IRF(ii,2:end),'-k','LineWidth',2);
            hold on
            plot(zeros(IRFsteps),'-r','LineWidth',.5)
            title(M_.endo_names(ii,:));
            xlim( [0 IRFsteps] )
        end
        suptitle('IRF Computed from Steady State')
        set(gcf, 'Color', 'w');
        figure_name = 'IRF_SteadyState';
        print(figure_name,'-dpdf'); print(figure_name,'-deps')


        
%% 3. Alternative IRF: compute deterministic simulation from a boom & from bust
%==========================================================================
% As above but instead of using the ergodic mean, use the steady state 
% as an initial value. RESULTS ARE MUCH BETTER! 

    case 3

        % Draw random shocks and simulate the model
        exopath = randn(steps,nexovar);
        out_baseline = simult_(oo_.steady_state,oo_.dr,exopath,3);

        % Find the boom in GDP (11th variable)
        select = out_baseline(11,:)==max(out_baseline(11,:));
        boom   = out_baseline(:, select);

        % Find the bust in GDP (11th variable)
        select = out_baseline(11,:)==min(out_baseline(11,:));
        bust   = out_baseline(:, select);
 
        % Initialize the path of the exogenous process 
        exopath_shock = zeros(IRFsteps,nexovar);

        % Set a 2 st. dev shock at period 1 for the chosen exogenous process
        exopath_shock(1,whichShock) = 2; % 2 st. dev. shock

        % Simulate the deterministic shock from the ergodic mean
        out1 = simult_(boom,oo_.dr,exopath_shock,3);
        out2 = simult_(bust,oo_.dr,exopath_shock,3);

        % Compute percent deviations from the mean
        for ii=1:nendovar
            IRF1(ii,:) = (out1(ii,2:end) - boom(ii));
            IRF2(ii,:) = (out2(ii,2:end) - bust(ii));
        end

        figure
        nplots = ceil(sqrt(nendovar));
        for ii=1:nendovar
            subplot(nplots,nplots,ii)
            plot(IRF1(ii,:),'-k','LineWidth',2);
            hold on
            plot(IRF2(ii,:),'--g','LineWidth',2);
            hold on
            plot(zeros(IRFsteps),'-r','LineWidth',.5)
            title(M_.endo_names(ii,:));
            xlim( [0 IRFsteps] )
            LegendSubplot({'End of Boom';'End of Bust'},0.3)
        end
        set(gcf, 'Color', 'w');
        figure_name = 'IRF_BoomBust';
        print(figure_name,'-dpdf'); print(figure_name,'-deps')

end


% close all

% %% Plots GDP level and cycle (from the simulated model)
% %==========================================================================
% figure
% out = oo_.endo_simul(11,:) ./ oo_.steady_state(11) -1;
% quarters = 2000;
% xAxis = [1:quarters];
% Level = cumsum(out(end-quarters+1:end)); 
% Cycle = out(end-quarters+1:end);
% 
% plotyy(xAxis,Level,xAxis,Cycle)
% title('GDP Level & Cycle')
