%This function unfolds the matrices reported by Dynare++ up to a third order approximation
%The assumption is that dynare++ .mat files are saved on the path, with '_#' indicating the 
%underlying approximation number.
%
% Our notation is as follows
% z_t = f([x_t-1;u_t])
% where 'x'    the state variables
%       'u'    the innovations 
% and v_t = [x_t-1;u_t]
%
% IMPORTANT: the codes are made for running DYNARE++ with the option --no-centralize
% By Martin M. Andreasen

function [f_ss,fv,fvv,fss,fvvv,fssv,fsss,sigma,nv,nu,nx,nz,dyn_vars,dyn_state_vars,dyn_shocks] ...
    = Dynare_Unfold_Matrices(model,order_app);

%Load model to initialise counter and aux_vars
load([model,'_1','.mat']);

%Auxiliary definitions
nv      = size(dyn_state_vars,1);   % Number of state variables and innovations
nu      = size(dyn_vcov_exo,1);     % Number of innovations
nx      = nv-nu;                    % Number of state variables
nz      = size(dyn_vars,1);         % Number of control variables + state variables

% The cholesky factorisation of the covariance matrix
[sigma,error_mes] = chol(dyn_vcov_exo);       
if error_mes ~= 0
    sigma = diag(sqrt(diag(dyn_vcov_exo)));
    disp('Calculaions done for diagonal shock covariance matrix')
end


%Load matrices with coefficients:
%Deterministic steady state
f_ss = dyn_ss;
%f_ij contains the matrix dyn_f_j corresponding to an i-th order approximation
for i = 1:order_app
    load([model,'_',num2str(i),'.mat']);
    for j = 0:i
        eval(['f_',num2str(i),num2str(j),'=dyn_g_',num2str(j),';']);
    end
end

%The first order terms
fv             = f_11;

% The second order terms 
% We multiply by 2 because the terms in Dynare++ are already multiplied by
% 1/2 (see below)
if order_app > 1
    fss            = 2*f_20;
    % we unfold the matrix f_22
    fvv            = zeros(nz,nv,nv);
    index = 0;
    for alfa1=1:nv
        for alfa2=alfa1:nv
            index = index + 1;
            fvv(:,alfa1,alfa2) =  f_22(:,index);
            if alfa1 ~= alfa2
               fvv(:,alfa2,alfa1) =  f_22(:,index);
            end
        end
    end
    fvv = 2*fvv;
else
    fss = zeros(nz,1);
    fvv = zeros(nz,nv,nv);
end

%The third order terms are given by f_33 and by the difference between f_31 and f_11
% We multiply by 6 because the terms in Dynare++ are already multiplied by
% 1/6 (see below)
if order_app > 2
    fssv           = (6/3)*(f_31-f_11);            %the term f_31 is multiplied by 3/6 in Dynare ++ because there are
                                                   %3 combinations (fssz,fszs,fzss) in these third order terms
    % We unfold the matrix f_33 and multiplt by 6
    fvvv           = zeros(nz,nv,nv,nv);
    index = 0;
    for alfa1=1:nv
        for alfa2=alfa1:nv
            for alfa3=alfa2:nv
                index = index + 1;
                fvvv(:,alfa1,alfa2,alfa3) = f_33(:,index);

                % Using symmetry for alfa1 and alfa2
                if alfa1 == alfa2 && alfa2 ~= alfa3 %alfa1==alfa2~=alfa3
                    %fvvv(:,alfa1,alfa1,alfa3)= fvvv(:,alfa1,alfa2,alfa3);
                    fvvv(:,alfa1,alfa3,alfa1) = fvvv(:,alfa1,alfa2,alfa3);
                    fvvv(:,alfa3,alfa1,alfa1) = fvvv(:,alfa1,alfa2,alfa3);                
                end
                % Using symmetry for alfa2 and alfa3            
                if alfa1 ~= alfa2 && alfa2 == alfa3  %alfa1~=alfa2==alfa3 
                    %fvvv(:,alfa1,alfa2,alfa2)= fvvv(:,alfa1,alfa2,alfa3);                
                    fvvv(:,alfa2,alfa1,alfa2) = fvvv(:,alfa1,alfa2,alfa3);                
                    fvvv(:,alfa2,alfa2,alfa1) = fvvv(:,alfa1,alfa2,alfa3);                                
                end            
                % Using symmetry for alfa1,alfa2, and alfa3            
                if alfa1 ~= alfa2 && alfa1 ~= alfa3 &&  alfa2 ~= alfa3 %alfa1~=alfa2~=alfa3
                    %fvvv(:,alfa1,alfa2,alfa3) = fvvv(:,alfa1,alfa2,alfa3);
                    fvvv(:,alfa1,alfa3,alfa2) = fvvv(:,alfa1,alfa2,alfa3);
                    fvvv(:,alfa3,alfa1,alfa2) = fvvv(:,alfa1,alfa2,alfa3);
                    fvvv(:,alfa3,alfa2,alfa1) = fvvv(:,alfa1,alfa2,alfa3);
                    fvvv(:,alfa2,alfa3,alfa1) = fvvv(:,alfa1,alfa2,alfa3); 
                    fvvv(:,alfa2,alfa1,alfa3) = fvvv(:,alfa1,alfa2,alfa3);                 
                end
            end
        end
    end
    fvvv = 6*fvvv;
    fsss = zeros(nz,1);    %Dynare++ solves only for Gaussian shocks
else
    fsss = zeros(nz,1);
    fssv = zeros(nz,nv);
    fvvv = zeros(nz,nv,nv,nv);
end

% ************************************************************************
% For the sake of completness, we here reproduce the way the information is
% stored in Dynare++ (a reply by Ondra K.)
%
% The rows of the matrices are clear, right? The ordering of rows is given by dyn_vars vector. 
% Columns determine with what deviations the number in the column is to be multiplied. 
% NOTE THAT the deviations are not from steady state, the deviations are from a fix-point, 
% which is saved as dyn_ss. So this means that you cannot find dyn_g_0, since it is zero, 
% since dyn_ss is chosen so that dyn_g_0 is zero (from the definition of the fix-point). 
% 
% The matrices are already mulitplied with the multipliers 1/2, 1/6 etc. 
% 
% The columns are ordered as folded indices of variables stored in the vector dyn_state. 
% You can find more details first in kord.pdf and then in tl.pdf downloadable from the documentation page. 
% Everything is described there. However, I provide a small example. Let a model have the states [K,C,EPS]. 
% If you assign indices to these variables in increasing order, say [K=0,C=1,EPS=2], 
% then the folded indices will be all growing sequences consisting from 0,1,2 in alphabetical ordering. 
% This is, for the third order: 
% 000 
% 001 
% 002 
% 011 
% 012 
% 022 
% 111 
% 112 
% 122 
% 222 
%
% for the fourth order: 
% 0000 
% 0001 
% 0002 
% 0011 
% 0012 
% 0022 
% 0111 
% 0112 
% 0122 
% 0222 
% 1111 
% 1112 
% 1122 
% 1222 
% 2222 
%
% By Martin M. Andreasen: Recall that for the second order terms, we need
% to multiply terms constructed from two different indices such as 01, 12, etc. by 2. 
% For third order terms constructed from 2 different indices such as 
% 001, 002, etc., we need to multiply these terms by 3. Finally, terms 
% constructed from three different indices like 012 must be multiplied by 6. 
